--
--  Copyright (C) 2001, 2002 MIT
--
--  This file is part of dotLRN.
--
--  dotLRN is free software; you can redistribute it and/or modify it under the
--  terms of the GNU General Public License as published by the Free Software
--  Foundation; either version 2 of the License, or (at your option) any later
--  version.
--
--  dotLRN is distributed in the hope that it will be useful, but WITHOUT ANY
--  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
--  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
--  details.
--

--
-- create portals model
--
-- @author <a href="mailto:arjun@openforce.net">arjun@openforce.net</a>
-- @author <a href="mailto:yon@openforce.net">yon@openforce.net</a>
-- @creation-date 2001-10-01
-- @version $Id: portal-core-create.sql,v 1.2 2003/09/25 02:12:16 vinodk Exp $
--

create sequence portal_seq;

create table portal_datasources (
    datasource_id                   integer
                                    constraint portal_datasources_pk
                                    primary key,
    name                            varchar(200)
                                    constraint p_datasources_name_nn
                                    not null,
    shadeable_p                     char(1)
                                    default 't'
                                    constraint p_datasources_shadeable_p_nn
                                    not null
                                    constraint p_datasources_shadeable_p_ck
                                    check (shadeable_p in ('t', 'f')),
    hideable_p                      char(1)
                                    default 't'
                                    constraint p_datasources_hideable_p_nn
                                    not null
                                    constraint p_datasources_hideable_p_ck
                                    check (hideable_p in ('t', 'f')),
    description                     varchar(200)
);

comment on table portal_datasources is '
';

create table portal_datasource_parameters (
    datasource_id                   integer
                                    constraint p_ds_params_datasource_id_fk
                                    references portal_datasources (datasource_id)
                                    on delete cascade
                                    constraint p_ds_params_datasource_id_nn
                                    not null,
    key                             varchar(200)
                                    constraint p_ds_params_key_nn
                                    not null,
    value                           varchar(200),
    config_required_p               char(1)
                                    default 'f'
                                    constraint p_ds_params_cfg_req_p_nn
                                    not null
                                    constraint p_ds_params_cfg_req_p_ck
                                    check (config_required_p in ('t', 'f')),
    configured_p                    char(1)
                                    default 'f'
                                    constraint p_ds_params_cfg_p_nn
                                    not null
                                    constraint p_ds_params_cfg_p_ck
                                    check (configured_p in ('t', 'f')),
    constraint portal_datasource_params_pk
    primary key (datasource_id, key)

);

comment on table portal_datasource_parameters is '
    maintains a set of default parameters for portal datsources. these
    are copied to a portal element when it is created.

    configuration semantics:

    config_required_p | configured_p      |
    ----------------- | ----------------- | ----------------------------------
            t         |         t         | static configuration is provided
                      |                   | for the portal elements. it can be
                      |                   | changed later.
    ----------------- | ----------------- | ----------------------------------
            t         |         f         | the portal element must be
                      |                   | configured when created.
    ----------------- | ----------------- | ----------------------------------
            f         |         t         | an optional default configuration
                      |                   | is provided.
    ----------------- | ----------------- | ----------------------------------
            f         |         f         | configuration is optional and no
                      |                   | default is provided.
';

comment on column portal_datasource_parameters.config_required_p is '
    this parameter requires configuration. if configured_p is true then no
    action is needed at portal element creation since the portal element
    can simply use the default value provided. if configured_p is false then
    the portal element must provide a value for this parameter at creation
    time.
';

comment on column portal_datasource_parameters.configured_p is '
    if true then a default value is provided for this parameter.
';

create table portal_layouts (
    layout_id                       integer
                                    constraint p_layouts_layout_id_fk
                                    references acs_objects (object_id)
                                    constraint portal_layouts_pk
                                    primary key,
    name                            varchar(200)
                                    constraint p_layouts_name_un
                                    unique
                                    constraint p_layouts_name_nn
                                    not null,
    description                     varchar(200),
    filename                        varchar(200),
    resource_dir                    varchar(200)
);

comment on table portal_layouts is '
';

comment on column portal_layouts.filename is '
';

comment on column portal_layouts.resource_dir is '
';

create table portal_layout_regions (
    layout_id                       integer
                                    constraint p_layout_regions_layout_id_fk
                                    references portal_layouts (layout_id)
                                    on delete cascade
                                    constraint p_layout_regions_layout_id_nn
                                    not null,
    region                          varchar(20)
                                    constraint p_layout_regions_region_nn
                                    not null,
    constraint portal_layout_regions_pk
    primary key (layout_id, region)
);

comment on table portal_layout_regions is '
';

comment on column portal_layout_regions.region is '
';

-- Themes are templates with decoration for PEs, nothing more.
-- At this point they will just be bits of ADPs  in the filesystem
create table portal_themes (
    theme_id                        integer
                                    constraint p_themes_theme_id_fk
                                    references acs_objects (object_id)
                                    constraint portal_themes_pk
                                    primary key,
    name                            varchar(200)
                                    constraint p_themes_name_un
                                    unique
                                    constraint p_themes_name_nn
                                    not null,
    description                     varchar(200),
    filename                        varchar(200),
    resource_dir                    varchar(200)
);

comment on table portal_themes is '
';

comment on column portal_themes.filename is '
';

comment on column portal_themes.resource_dir is '
';

-- Portals are essentially "containers" for PEs that bind to DSs.
-- Parties have, optionally have portals
-- Restrict to party check?
-- Roles and perms issues?
create table portals (
    portal_id                       integer
                                    constraint p_portal_id_fk
                                    references acs_objects (object_id)
                                    constraint portals_pk
                                    primary key,
    name                            varchar(200)
                                    default 'Untitled'
                                    constraint p_name_nn
                                    not null,
    theme_id                        integer
                                    constraint p_theme_id_fk
                                    references portal_themes (theme_id)
                                    constraint p_theme_id_nn
                                    not null,
    template_id                     integer
                                    constraint p_template_id_fk
                                    references portals (portal_id)
);

comment on table portals is '
';

comment on column portals.template_id is '
';

create table portal_pages (
    page_id                         integer
                                    constraint p_pages_page_id_fk
                                    references acs_objects (object_id)
                                    constraint portal_pages_pk
                                    primary key,
    name                            varchar(200)
                                    default 'Untitled Page'
                                    constraint p_pages_name_nn
                                    not null,
    portal_id                       integer
                                    constraint p_pages_portal_id_fk
                                    references portals (portal_id)
                                    constraint p_pages_portal_id_nn
                                    not null,
    layout_id                       integer
                                    constraint p_pages_layout_id_fk
                                    references portal_layouts (layout_id)
                                    constraint p_pages_layout_id_nn
                                    not null,
    sort_key                        integer
                                    constraint p_pages_sort_key_nn
                                    not null,
    constraint p_pages_portal_id_sort_key_un
    unique (portal_id, sort_key)
);

create index portal_pages_prtl_page_idx on portal_pages (portal_id, page_id);

comment on table portal_pages is '
';

comment on column portal_pages.sort_key is '
';

-- Support for multi-page portals (think my.yahoo.com)
-- PE are fully owned by one and only one portal. They are not
-- "objects" that live on after their portal is gone. One way to think
-- of them is a map b/w a portal and a DS, with satellite data of a
-- theme, a config, a region, etc.
--
-- No securtiy checks are done here. If you can view and bind to a DS you have
-- a PE for it.

create table portal_elements (
    element_id                      integer
                                    constraint portal_elements_pk
                                    primary key,
    datasource_id                   integer
                                    constraint p_elements_datasource_id_fk
                                    references portal_datasources (datasource_id)
                                    on delete cascade
                                    constraint p_elements_datasource_id_nn
                                    not null,
    name                            varchar(200)
                                    constraint p_elements_name_nn
                                    not null,
    page_id                         integer
                                    constraint p_elements_page_id_fk
                                    references portal_pages
                                    on delete cascade
                                    constraint p_elements_page_id_nn
                                    not null,
    region                          varchar(20)
                                    constraint p_elements_region_nn
                                    not null,
    sort_key                        integer
                                    constraint p_elements_sort_key_nn
                                    not null
                                    constraint p_elements_sort_key_ck
                                    check (sort_key > 0),
    state                           varchar(6)
                                    default 'full'
                                    constraint p_elements_state_ck
                                    check (state in ('full', 'shaded', 'hidden', 'pinned')),
    shadeable_p                     char(1)
                                    default 't'
                                    constraint p_elements_shadeable_p_nn
                                    not null
                                    constraint p_elements_shadeable_p_ck
                                    check (shadeable_p in ('t', 'f')),
    hideable_p                      char(1)
                                    default 't'
                                    constraint p_elements_hideable_p_nn
                                    not null
                                    constraint p_elements_hideable_p_ck
                                    check (hideable_p in ('t', 'f')),
    constraint p_elements_page_id_name_un
    unique (page_id, name)
);

comment on table portal_elements is '
';

comment on column portal_elements.sort_key is '
';

comment on column portal_elements.state is '
';

create table portal_element_parameters (
    parameter_id                    integer
                                    constraint portal_element_parameters_pk
                                    primary key,
    element_id                      integer
                                    constraint p_element_params_element_id_fk
                                    references portal_elements (element_id)
                                    on delete cascade
                                    constraint p_element_params_element_id_nn
                                    not null,
    key                             varchar(50)
                                    constraint p_element_params_key_nn
                                    not null,
    value                           varchar(200),
    config_required_p               char(1)
                                    default 'f'
                                    constraint p_element_params_cfg_req_p_nn
                                    not null
                                    constraint p_element_params_cfg_req_p_ck
                                    check (config_required_p in ('t', 'f')),
    configured_p                    char(1)
                                    default 'f'
                                    constraint p_element_params_cfg_p_nn
                                    not null
                                    constraint p_element_params_cfg_p_ck
                                    check (configured_p in ('t', 'f'))
);

create index p_element_params_element_key_idx on portal_element_parameters (element_id, key);

comment on table portal_element_parameters is '
';

comment on column portal_element_parameters.config_required_p is '
';

comment on column portal_element_parameters.configured_p is '
';
