#
#  Copyright (C) 2001, 2002 MIT
#
#  This file is part of dotLRN.
#
#  dotLRN is free software; you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation; either version 2 of the License, or (at your option) any later
#  version.
#
#  dotLRN is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#

ad_library {

    portal procs

    @author <a href="mailto:arjun@openforce.net">arjun@openforce.net</a>
    @author <a href="mailto:yon@openforce.net">yon@openforce.net</a>
    @creation-date 2001-09-01
    @version $Id: portal-procs.tcl,v 1.1.1.1 2002/10/25 21:29:17 yon Exp $

}

namespace eval portal {

    ad_proc -private package_key {} {
        returns the package_key
    } {
        return "portal"
    }

    ad_proc -public get_package_id {} {
        returns the package_id
    } {
        return [apm_package_id_from_key [package_key]]
    }

    # work around for template::util::url_to_file
    ad_proc -private www_path {} {
        returns the path of the www dir of the portal package. we need this for
        stupid template tricks.
    } {
        return "/packages/[package_key]/www"
    }

    ad_proc -public mount_point {} {
        caches the mount point
    } {
        return [lindex [site_node::get_url_from_object_id -object_id [get_package_id]] 0]
    }

    ad_proc -public automount_point {} {
        packages such as dotlrn can automount the portal here
    } {
        return portal
    }

    ad_proc -public new {
        {-party_id:required}
        {-name Untitled}
        {-template_id ""}
        {-context_id ""}
        {-page_list {{{Page 1} 2_column}}}
    } {
        create a new portal for the passed in party_id. create pages passed in
        the page_list.
    } {
        db_transaction {

            set extra_vars [ns_set create]
            ns_set put $extra_vars name $name
            ns_set put $extra_vars template_id $template_id
            ns_set put $extra_vars context_id $context_id

            set portal_id [package_instantiate_object -extra_vars $extra_vars portal]

            permission::grant -party_id $party_id -object_id $portal_id -privilege read
            permission::grant -party_id $party_id -object_id $portal_id -privilege write
            permission::grant -party_id $party_id -object_id $portal_id -privilege admin

            if {[empty_string_p $template_id]} {
                foreach page $page_list {
                    portal::page::new \
                        -portal_id $portal_id \
                        -name [lindex $page 0] \
                        -layout_name [lindex $page 1]
                }
            }

        }

        return $portal_id
    }

    ad_proc -public delete {
        {-portal_id:required}
    } {
        delete a portal
    } {
        db_dml delete_permissions {}
        db_exec_plsql delete_portal {}
    }

    ad_proc -public get {
        {-portal_id:required}
    } {
        get portal info in "array get" format (cached)
    } {
        return [util_memoize "portal::get_not_cached -portal_id $portal_id"]
    }

    ad_proc -private get_not_cached {
        {-portal_id:required}
    } {
        get portal info in "array get" format (not cached)
    } {
        db_1row select_portal_info {} -column_array portal
        return [array get portal]
    }

    ad_proc -public set_values {
        {-portal:required}
    } {
        set the fields of a portal
    } {
        array set portal_array $portal
        template::util::array_to_vars portal

        db_dml update_portal {}
        util_memoize_flush "portal::get_not_cached -portal_id $portal_id"
    }

    ad_proc -public get_name {
        {-portal_id:required}
    } {
        array set portal [get -portal_id $portal_id]
        return $portal(name)
    }

    ad_proc -public set_name {
        {-portal_id:required}
        {-name:required}
    } {
        array set portal [get -portal_id $portal_id]
        set portal(name) $name

        set_values -portal [array get portal]
    }

    ad_proc -public get_theme_id {
        {-portal_id:required}
    } {
        array set portal [get -portal_id $portal_id]
        return $portal(theme_id)
    }

    ad_proc -public set_theme_id {
        {-portal_id:required}
        {-theme_id:required}
    } {
        array set portal [get -portal_id $portal_id]
        set portal(theme_id) $theme_id

        set_values -portal [array get portal]
    }

    ad_proc -private get_template_id {
        {-portal_id:required}
    } {
        array set portal [get -portal_id $portal_id]
        return $portal(template_id)
    }

    ad_proc -public get_page_count {
        {-portal_id:required}
    } {
        return [db_string select_page_count {}]
    }

    ad_proc -public get_page_list {
        {-portal_id:required}
    } {
        @return list of page_ids associated with the given portal (in sort_key order).
    } {
        return [db_list select_page_ids {}]
    }

    ad_proc -public render {
        {-portal_id:required}
        {-page_num 0}
        {-render_style individual}
    } {
        returns a chunk of html that is a fully rendered portal
    } {
        permission::require -privilege read -object_id $portal_id
        set edit_p [permission::permission_p -object_id $portal_id -privilege write]

        set master_template [parameter::get -parameter master_template]

        # get the portal and layout
        db_1row portal_select {} -column_array portal
        set page_id $portal(page_id)

        set element_ids [list]
        db_foreach element_select {} -column_array entry {
            lappend element_ids($entry(region)) $entry(element_id)
        }

        set element_list [array get element_ids]

        # set up the template, it includes the layout template,
        # which in turn includes the theme, then elements
        if {[empty_string_p $element_list]} {
            # The portal has no elements, show anyway (they can configure)
            set template "<master src=\"@master_template@\">
            <property name=\"title\">@portal.name@</property>"
        } else {
            set element_src "[www_path]/render-styles/${render_style}/render-element"
            set template "<master src=\"@master_template@\">
            <property name=\"title\">@portal.name@</property>
            <include src=\"@portal.layout_filename@\"
            element_list=\"@element_list@\"
            element_src=\"@element_src@\"
            theme_id=@portal.theme_id@
            portal_id=@portal.portal_id@
            edit_p=@edit_p@
            page_id=@page_id@
            layout_id=@portal.layout_id@>"
        }

        # necessary hack to work around the acs-templating system
        set __adp_stub "[get_server_root][www_path]/."
        set {master_template} \"master\"

        # compile and evaluate the template
        set code [template::adp_compile -string $template]
        return [template::adp_eval code]
    }

    ad_proc -private layout_elements {
        element_list
        {var_stub "element_ids"}
    } {
        Split a list up into a bunch of variables for inserting into a
        layout template. This seems pretty kludgy (probably because it is),
        but a template::multirow isn't really well suited to data of this
        shape. It'll setup a set of variables, $var_stub_1 - $var_stub_8
        and $var_stub_i1- $var_stub_i8, each contining the portal_ids that
        belong in that region. - Ian Baker

        @param element_id_list An [array get]'d array, keys are regions, \
                values are lists of element_ids.
        @param var_stub A name upon which to graft the bits that will be \
                passed to the template.
    } {
       array set elements $element_list

        foreach idx [list 1 2 3 4 5 6] {
            upvar "${var_stub}_$idx" group
            if { [info exists elements($idx) ] } {
                set group $elements($idx)
            } else {
                set group {}
            }
        }
    }

    ad_proc -public configure {
        {-referer ""}
        {-template_p f}
        portal_id
        return_url
    } {
        Return a portal or portal template configuration page.
        All form targets point to file_stub-2.

        FIXME REFACTOR ME

        @param portal_id
        @return_url
    } {
        set edit_p [permission::permission_p -object_id $portal_id -privilege write]

        if {!$edit_p} {
            permission::require -privilege admin -object_id $portal_id
            set edit_p 1
        }

        #
        # Set up some whole page stuff
        #

        set master_template [ad_parameter master_template]
        set action_string [generate_action_string]
        set element_src "[portal::www_path]/place-element"

        if {[empty_string_p $referer]} {
            set return_text "<a href=@return_url@>Go back</a>"
        } else {
            set return_text ""
            set return_url $referer
        }

        #
        # Begin creating the template
        #

        set template "
        <master src=\"@master_template@\">
        <p>
        $return_text
        <p>"

        #
        # Theme selection chunk
        #

        set theme_chunk "
            <form method=post action=@action_string@>
            <input type=hidden name=portal_id value=@portal_id@>
            <input type=hidden name=return_url value=@return_url@>
            <strong>Change Theme:</strong>
            <br>"
        set current_theme_id [portal::get_theme_id -portal_id $portal_id]

        foreach theme [portal::theme::get_list] {
            set theme_id [ns_set get $theme theme_id]
            set name [ns_set get $theme name]
            set description [ns_set get $theme description]

            append theme_chunk "<label><input type=radio name=theme_id value=$theme_id"
            set one_theme_chunk "&nbsp;$name - $description"

            if {$current_theme_id == $theme_id } {
                append theme_chunk " checked><b>$one_theme_chunk</b>"
            } else {
                append theme_chunk ">$one_theme_chunk"
            }
            append theme_chunk "</label><br>\n"
        }

        append theme_chunk "<input type=submit name=op value=\"Change Theme\"></form>"
        append template "$theme_chunk"

        #
        # Per-page template chunks
        #

        set list_of_page_ids [get_page_list -portal_id $portal_id]

        foreach page_id $list_of_page_ids {

            set first_page_p [portal::first_page_p -portal_id $portal_id -page_id $page_id]
            set page_name [portal::page::get_name -page_id $page_id]
            set page_layout_id [portal::page::get_layout_id -page_id $page_id]

            append template "<table bgcolor=#eeeeee border=0 width=\"100%\">"

            #
            # Page rename chunk
            #

            set page_name_chunk "
            <a name=$page_id></a>
            <form method=post action=@action_string@>
            <input type=hidden name=portal_id value=@portal_id@>
            <input type=hidden name=page_id value=$page_id>
            <input type=hidden name=return_url value=@return_url@>
            <input type=hidden name=anchor value=$page_id>
            <br><strong>Page:</strong>
            <input type=text name=name value=\"$page_name\">
            <input type=submit name=op value=\"Rename Page\">
            </form>"
            append template "$page_name_chunk"

            if {[portal::page::has_visible_elements -page_id $page_id] || $first_page_p} {

                #
                # Page with non-hidden elements OR the first page of the portal
                #

                db_1row portal_and_page_info_select {} -column_array portal

                # fake some elements for the <list> in the template
                foreach region [portal::layout::get_region_list -layout_id $portal(layout_id)] {
                    lappend fake_element_ids($region) $portal_id
                }

                set element_list [array get fake_element_ids]

                append template "
                <table bgcolor=#eeeeee border=0 width=\"100%\">
                <tr valign=middle><td valign=middle>
                <include src=\"$portal(template)\"
                element_list=\"$element_list\"
                action_string=@action_string@ portal_id=@portal_id@
                return_url=\"@return_url@\" element_src=\"@element_src@\"
                page_id=$page_id
                layout_id=$portal(layout_id)
                edit_p=@edit_p@>
                </td></tr>"

                # clear out the region array
                array unset fake_element_ids
            }

            if {![portal::has_visible_elements -page_id $page_id]} {

                #
                # Non first page with all hidden elements
                #

                #
                # Remove page chunk - don't allow removal of the first page
                #

                if {!$first_page_p} {
                    append template "
                    <tr valign=middle><td valign=middle>
                    <center>
                    No Elements on this page
                    <form method=post action=@action_string@>
                    <input type=hidden name=portal_id value=$portal_id>
                    <input type=hidden name=page_id value=$page_id>
                    <input type=hidden name=return_url value=@return_url@>
                    <input type=submit name=op value=\"Remove Empty Page\">
                    </form>
                    </center>
                    </td>
                    </tr>"
                }

                #
                # Layout change chunk - only shown when there are no visible elements on the page
                #

                set layout_chunk ""

                foreach layout [portal::layout::get_layout_list] {
                    set layout_id [ns_set get $layout layout_id]
                    set layout_name [ns_set get $layout layout_name]
                    set layout_description [ns_set get $layout layout_description]
                    set one_layout_chunk "<small>&nbsp;$layout_name - $layout_description</small>"
                    append layout_chunk "<label><input type=radio name=layout_id value=$layout_id"

                    if {$page_layout_id == $layout_id} {
                        append layout_chunk " checked><b>$one_layout_chunk</b>"
                    } else {
                        append layout_chunk ">$one_layout_chunk"
                    }

                    append layout_chunk "</label><br>\n"
                }


                append template "
                <tr>
                <td>
                <br>
                <form method=post action=@action_string@>
                <small>
                <b>Change page layout:</b>
                <br>
                <input type=hidden name=portal_id value=$portal_id>
                <input type=hidden name=page_id value=$page_id>
                <input type=hidden name=return_url value=@return_url@>
                $layout_chunk
                <input type=submit name=op value=\"Change Page Layout\">
                </small>
                </form>
                </td></tr>"

            }


            # close the page's table
            append template "</table>"

        }

        #
        # New page chunk
        #

        set new_page_num [expr [get_page_count -portal_id $portal_id] + 1]

        append template "
        <a name=add_a_new_page></a>
        <form method=post action=@action_string@>
        <input type=hidden name=portal_id value=@portal_id@>
        <input type=hidden name=return_url value=@return_url@>
        <input type=hidden name=anchor value=add_a_new_page>
        <b>Add a new page:</b>
         <input type=text name=name value=\"Page $new_page_num\">
        <input type=submit name=op value=\"Add Page\">
        </form>
        "

        #
        # Revert page chunk
        #

        if {![empty_string_p [get_template_id -portal_id $portal_id]]} {
            append template "
            <br>
            <form method=post action=@action_string@>
            <input type=hidden name=portal_id value=@portal_id@>
            <input type=hidden name=return_url value=@return_url@>
            <b>Revert to the default arrangement:</b>
            <input type=submit name=op value=\"Revert\">
            </form>"
        }

        #
        # Templating system hacks
        #

        set __adp_stub "[get_server_root][www_path]/."
        set {master_template} \"master\"

        set code [template::adp_compile -string $template]
        set output [template::adp_eval code]

        return $output
    }

    ad_proc -public configure_dispatch {
        {-template_p f}
        {-portal_id:required}
        {-form:required}
    } {
        Dispatches the configuration operation.
        We get the target region number from the op.

        @param portal_id the portal to edit
        @param formdata an ns_set with all the formdata
    } {

        set edit_p [permission::permission_p -object_id $portal_id -privilege write]

        if {!$edit_p} {
            permission::require -privilege admin -object_id $portal_id
            set edit_p 1
        }

        set op [ns_set get $form op]

        switch $op {
            "Revert" {
                db_transaction {
                    set template_id [get_template_id -portal_id $portal_id]

                    # revert theme
                    set_theme_id -portal_id $portal_id -theme_id [get_theme_id -portal_id $template_id]

                    # revert pages
                    # first equalize number of pages in the target
                    set template_page_count [get_page_count -portal_id $template_id]
                    set target_page_count [get_page_count -portal_id $portal_id]
                    set difference [expr $template_page_count - $target_page_count]

                    if {$difference > 0} {
                        # less pages in target
                        for {set x 0} {$x < $difference} {incr x} {

                            set name "portal revert dummy page $x"
                            portal::page::new \
                                -portal_id $portal_id \
                                -name $name \
                        }
                    } elseif {$difference < 0} {
                        # more pages in target, delete them from the end,
                        # putting any elements on them on the first page,
                        # we put them in the right place later
                        for {set x 0} {$x < [expr abs($difference)]} {incr x} {

                            set max_page_id [db_string revert_max_page_id_select {}]
                            set page_id [db_string revert_min_page_id_select {}]
                            set region 1

                            db_foreach revert_move_elements_for_del {} {
                                portal::move_element_to_page \
                                    -page_id $page_id \
                                    -element_id $element_id \
                                    -region 1
                            }

                            portal::page::delete -page_id $max_page_id
                        }
                    }

                    # now that they have the same number of pages, get to it
                    foreach source_page_id [get_page_list -portal_id $template_id] {

                        db_1row revert_get_source_page_info {}

                        set target_page_id [db_string revert_get_target_page_id {}]

                        db_dml revert_page_update {}

                        # revert elements in two steps like "swap"
                        db_foreach revert_get_source_elements {} {
                            # the element might not be on the target page...
                            set target_element_id \
                                    [db_string revert_get_target_element {}]

                            db_dml revert_element_update {}
                        }
                    }
                }
            }
            Rename {
                portal::set_name \
                    -portal_id $portal_id \
                    -name [ns_set get $form new_name]
            }
            swap -
            move {
                portal::element::move \
                    -element_id [ns_set get $form element_id] \
                    -direction [ns_set get $form direction]
            }
            "Show Here" {
                set region [ns_set get $form region]
                set element_id [ns_set get $form element_id]
                set page_id [ns_set get $form page_id]

                db_transaction {
                    # The new element's sk will be the last in the region
                    db_dml show_here_update_sk {}
                    db_dml show_here_update_state {}
                }
            }
            "Move to page" {
                portal::element::move_to_page \
                    -page_id [ns_set get $form page_id] \
                    -element_id [ns_set get $form element_id]
            }
            hide {
                set element_id_list [list]

                # iterate through the set, destructive!
                while { [expr [ns_set find $form "element_id"] + 1 ] } {
                    lappend element_id_list [ns_set get $form "element_id"]
                    ns_set delkey $form "element_id"
                }

                if {! [empty_string_p $element_id_list] } {
                    db_transaction {
                        foreach element_id $element_id_list {
                            db_dml hide_update {}

                            # after hiding an element, add
                            # it to the _first_ page
                            # of the portal.
                            portal::move_element_to_page \
                                -page_id [portal::get_page_id -portal_id $portal_id] \
                                -element_id $element_id
                        }
                    }
                }
            }
            "Change Theme" {
                set_theme_id -portal_id $portal_id -theme_id [ns_set get $form theme_id]
            }
            "Add Page" {
                set name [ns_set get $form name]
                if {[empty_string_p $name]} {
                    ad_return_complaint 1 "You must enter a name for the new page."
                }
                portal::page::new -portal_id $portal_id -name $name
            }
            "Remove Empty Page" {
                portal::page::delete -page_id [ns_set get $form page_id]
            }
            "Change Page Layout" {
                portal::page::set_layout_id \
                    -page_id [ns_set get $form page_id] \
                    -layout_id [ns_set get $form layout_id]
            }
            "Rename Page" {
                set name [ns_set get $form name]
                set page_id [ns_set get $form page_id]

                if {[empty_string_p $name]} {
                    ad_return_complaint 1 "You must enter new name for the page."
                }
                portal::page::set_name -page_id $page_id -name $name
            }
            toggle_pinned {
                set element_id [ns_set get $form element_id]

                if {[string equal [portal::element::get_state -element_id $element_id] full]} {
                    portal::element::pin -element_id $element_id
                } else {
                    portal::element::unpin -element_id $element_id
                }
            }
            toggle_shadeable {
                set element_id [ns_set get $form element_id]

                if {[string equal [portal::element::get_shadeable_p -element_id $element_id] f]} {
                    portal::element::set_shadeable_p -element_id $element_id -shadeable_p t
                } else {
                    portal::element::set_shadeable_p -element_id $element_id -shadeable_p f
                }
            }
            toggle_hideable {
                set element_id [ns_set get $form element_id]

                if {[string equal [portal::element::get_hideable_p -element_id $element_id] f]} {
                    portal::element::set_hideable_p -element_id $element_id -hideable_p t
                } else {
                    portal::element::set_hideable_p -element_id $element_id -hideable_p f
                }
            }
            default {
                ns_log error  "portal::configure_dispatch: bad op = $op!"
                ad_return_complaint 1 "portal::configure_dispatch: bad op!\nop = $op"
            }
        }
    }

    #
    # portal template procs - util and configuration
    #

    ad_proc -public template_configure {
        portal_id
        return_url
    } {
        Just a wrapper for the configure proc.

        @param portal_id
        @return A portal configuration page
    } {
        portal::configure -template_p "t" $portal_id $return_url
    }

    ad_proc -public template_configure_dispatch {
        portal_id
        form
    } {
        Just a wrapper for the configure_dispatch proc

        @param portal_id
        @param formdata an ns_set with all the formdata
    } {
        configure_dispatch -template_p "t" $portal_id $form
    }

    ad_proc -public navbar {
        {-portal_id:required}
        {-td_align "left"}
        {-link ""}
        {-pre_html ""}
        {-post_html ""}
        {-link_all 0}
        {-extra_td_html ""}
        {-table_html_args ""}
    } {
        wraps portal::dimensional to create a dotlrn navbar

        @return the id of the page
        @param portal_id
        @param link the relative link to set for hrefs
        @param current_page_link f means that there is no link for the current page
    } {
        set ad_dim_struct [list]

        db_foreach list_page_nums_select {} {
            lappend ad_dim_struct [list $page_num $name [list]]
        }

        set ad_dim_struct "{ page_num \"Page:\" 0 [list $ad_dim_struct] }"

        return [dimensional -no_header \
                -no_bars \
                -link_all $link_all \
                -td_align $td_align \
                -pre_html $pre_html \
                -post_html $post_html \
                -extra_td_html $extra_td_html \
                -table_html_args $table_html_args \
                $ad_dim_struct \
                $link]
    }

    ad_proc -private generate_action_string {
    } {
        Portal configuration pages need this to set up
        the target for the generated links. It's just the
        current location with "-2" appended to the name of the
        page.
    } {
        return "[lindex [ns_conn urlv] [expr [ns_conn urlc] - 1]]-2"
    }

    ad_proc -public show_proc_helper {
        {-template_src ""}
        {-package_key:required}
        {-config_list:required}
    } {
        hides ugly templating calls for portlet "show" procs
    } {

        if { $template_src == ""} {
            set template_src $package_key
        }

        # some stupid upvar tricks to get them set right
        upvar __ts ts
        set ts $template_src

        upvar __pk pk
        set pk $package_key

        upvar __cflist cflist
        set cflist $config_list

        uplevel 1 {
            set template "<include src=\"$__ts\" cf=\"$__cflist\">"
            set __adp_stub "[get_server_root]/packages/$__pk/www/."
            set code [template::adp_compile -string $template]
            set output [template::adp_eval code]
            return $output
        }
    }

    ad_proc dimensional {
        {-no_header:boolean}
        {-no_bars:boolean}
        {-link_all 0}
        {-th_bgcolor "#ECECEC"}
        {-td_align "center"}
        {-extra_td_html ""}
        {-table_html_args "border=0 cellspacing=0 cellpadding=3 width=100%"}
        {-pre_html ""}
        {-post_html ""}
        option_list
        {url {}}
        {options_set ""}
        {optionstype url}
    } {
        An enhanced ad_dimensional. see that proc for usage details
    } {
        if {[empty_string_p $option_list]} {
            return
        }

        if {[empty_string_p $options_set]} {
            set options_set [ns_getform]
        }

        if {[empty_string_p $url]} {
            set url [ad_conn url]
        }

        set html "\n<table $table_html_args>\n <tr>\n"

        if {!$no_header_p} {
            foreach option $option_list {
                append html "    <th bgcolor=\"$th_bgcolor\">[lindex $option 1]</th>\n"
            }
        }

        append html "  </tr>\n  <tr>\n"

        foreach option $option_list {
            append html "    <td align=$td_align>"

            if {!$no_bars_p} {
                append html "\["
            }

            # find out what the current option value is.
            # check if a default is set otherwise the first value is used
            set option_key [lindex $option 0]
            set option_val [lindex $option 2]
            if {![empty_string_p $options_set]} {
                set option_val [ns_set get $options_set $option_key]
            }

            set first_p 1
            foreach option_value [lindex $option 3] {
                set thisoption_name [lindex $option_value 0]
                set thisoption_value [lindex $option_value 1]
                set thisoption_link_p 1
                if {[llength $option_value] > 3} {
                    set thisoption_link_p [lindex $option_value 3]
                }

                if {$first_p} {
                    set first_p 0
                } else {
                    if {!$no_bars_p} {
                        append html " | "
                    } else {
                        append html " &nbsp; "
                    }
                }

                if {([string equal $option_val $thisoption_name] == 1 && !$link_all) || !$thisoption_link_p} {
                    append html "${pre_html}<strong>${thisoption_value}</strong>${post_html}"
                } else {
                    append html "<a href=\"$url?[export_ns_set_vars url $option_key $options_set]&[ns_urlencode $option_key]=[ns_urlencode $thisoption_name]\">${pre_html}${thisoption_value}${post_html}</a>"
                }
            }

            if {!$no_bars_p} {
                append html "\]"
            }

            append html "$extra_td_html</td>\n"
        }

        append html "  </tr>\n</table>\n"
    }

}
